from django.core.management.base import BaseCommand

from ... import facade


class Command(BaseCommand):
    help = 'Close all inactive card-accounts'

    def handle(self, *args, **options):
        facade.close_expired_accounts()



from django.db import models
from django.utils import timezone
from polymorphic.managers import PolymorphicManager


class ActiveAccountManager(PolymorphicManager):
    def get_queryset(self):
        now = timezone.now()
        qs = super().get_queryset()
        return qs.filter(models.Q(start_date__lte=now) | models.Q(start_date=None)).filter(
            models.Q(end_date__gte=now) | models.Q(end_date=None)
        )


class ExpiredAccountManager(PolymorphicManager):
    def get_queryset(self):
        now = timezone.now()
        qs = super().get_queryset()
        return qs.filter(end_date__lt=now)


class TransactionManager(PolymorphicManager):
    pass



class AccountException(Exception):
    pass


class AccountNotEmpty(AccountException):
    pass


class InsufficientFunds(AccountException):
    pass


class InvalidAmount(AccountException):
    pass


class InactiveAccount(AccountException):
    pass


class ClosedAccount(AccountException):
    pass



from django.core.management.base import BaseCommand

from ...setup import create_default_accounts


class Command(BaseCommand):
    help = "Initialize baristand accounts default structure"

    def handle(self, *args, **options):
        create_default_accounts()



from django.core.management.base import BaseCommand

from ...models import Account


class Command(BaseCommand):
    help = "Update recalculate account balance"

    def handle(self, *args, **options):
        for acc in Account.objects.all():
            acc.save()



from django.db import transaction

from simpel_hookup import core as hookup

from simpel_journals.models import AccountType

from .settings import journals_settings

names = journals_settings.NAMES


@transaction.atomic
def create_default_accounts():
    """Create the default structure"""
    get_or_create_type = AccountType.objects.get_or_create
    asset, _ = get_or_create_type(name=names["ASSET"])
    asset.save()
    cash, _ = get_or_create_type(parent=asset, name=names["CASH"])
    cash.save()
    petty_cash, _ = get_or_create_type(parent=cash, name=names["PETTY_CASH"])
    petty_cash.save()
    bank_cash, _ = get_or_create_type(parent=cash, name=names["BANK"])
    bank_cash.save()

    account_receivable, _ = get_or_create_type(parent=asset, name=names["ACCOUNT_RECEIVABLE"])
    account_receivable.save()

    liabilities, _ = get_or_create_type(name=names["LIABILITY"])
    liabilities.debit = AccountType.DECREASE
    liabilities.save()

    account_payable, _ = get_or_create_type(parent=liabilities, name=names["ACCOUNT_PAYABLE"])
    account_payable.debit = AccountType.DECREASE
    account_payable.save()

    partner_balance, _ = get_or_create_type(parent=liabilities, name=names["PARTNER_BALANCE"])
    partner_balance.debit = AccountType.DECREASE
    partner_balance.save()

    revenue, _ = get_or_create_type(name=names["REVENUE"])
    revenue.debit = AccountType.DECREASE
    revenue.save()

    expenses, _ = get_or_create_type(name=names["EXPENSE"])
    expenses.save()

    # Invoke Hooked Account Initialization
    funcs = hookup.get_hooks("REGISTER_INITIAL_ACCOUNTS")
    for func in funcs:
        func()



from django.apps import AppConfig
from django.conf import settings
from django.db.models.signals import post_migrate
from django.utils.translation import gettext_lazy as _


class SimpelJournalsConfig(AppConfig):
    icon = "cash-100"
    default_auto_field = "django.db.models.BigAutoField"
    name = "simpel_journals"
    app_label = "simpel_journals"
    verbose_name = "Journals"

    def ready(self):
        from . import signals  # NOQA
        post_migrate.connect(init_app, sender=self)

def init_app(sender, **kwargs):
    pass



from decimal import Decimal

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType  # NOQA
from django.db import models, transaction
from django.db.models import Sum
from django.forms import ValidationError
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from mptt.models import MPTTModel, TreeForeignKey
from polymorphic.managers import PolymorphicManager
from polymorphic.models import PolymorphicModel
from simpel_numerators.models import NumeratorMixin, NumeratorReset

from simpel_utils.models.fields import CustomGenericForeignKey

from .managers import ActiveAccountManager, ExpiredAccountManager, TransactionManager

LEN_SHORT = 128
LEN_LONG = 255


class AccountType(MPTTModel):
    INCREASE = 1
    DECREASE = -1
    DEBITS = (
        (INCREASE, _("Increase")),
        (DECREASE, _("Decrease")),
    )
    parent = TreeForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="children",
    )
    debit = models.IntegerField(
        choices=DEBITS,
        default=INCREASE,
    )
    code = models.CharField(
        max_length=128,
        unique=True,
        null=True,
        blank=True,
    )
    name = models.CharField(
        max_length=255,
    )

    class Meta:
        db_table = "simpel_accounting_account_type"
        verbose_name = _("Account Type")
        verbose_name_plural = _("Account Types")

    def __str__(self):
        return self.name

    @property
    def full_name(self):
        names = [a.name for a in self.get_ancestors()]
        names.append(self.name)
        return " / ".join(names)

    def clean(self):
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError("Parent type cannot be self.")
            if parent.parent and parent.parent == self:
                raise ValidationError("Cannot have circular Parents.")

    def save(self, *args, **kwargs):
        parent = getattr(self, "parent", None)
        if parent:
            self.debit = parent.debit
        return super().save(*args, **kwargs)


class Account(PolymorphicModel):

    account_type = TreeForeignKey(
        AccountType,
        models.CASCADE,
        related_name="accounts",
        null=True,
    )
    created_at = models.DateTimeField(
        null=True,
        blank=True,
        editable=False,
        verbose_name=_("Created at"),
    )

    last_modified_at = models.DateTimeField(
        null=True,
        blank=True,
        editable=False,
        verbose_name=_("Created at"),
    )

    start_date = models.DateTimeField(
        null=True,
        blank=True,
    )
    end_date = models.DateTimeField(
        null=True,
        blank=True,
    )

    code = models.CharField(
        max_length=128,
        unique=True,
        null=True,
        blank=True,
    )
    linked_object_type = models.ForeignKey(
        ContentType,
        related_name="accounts",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        help_text=_("Linked instance type"),
    )
    linked_object_id = models.IntegerField(null=True, blank=True, help_text=_("Linked instance primary key."))
    linked_object = GenericForeignKey("linked_object_type", "linked_object_id")

    name = models.CharField(
        max_length=128,
        null=True,
        blank=True,
    )
    description = models.TextField(
        null=True,
        blank=True,
        help_text=_("This text is shown to customers during checkout"),
    )

    # Track the status of a account - this is often used so that expired
    # account can have their money transferred back to some parent account and
    # then be closed.
    OPEN, FROZEN, CLOSED = "Open", "Frozen", "Closed"
    STATUS = (
        (OPEN, _("Open")),
        (FROZEN, _("Frozen")),
        (CLOSED, _("Closed")),
    )
    status = models.CharField(max_length=32, choices=STATUS, default=OPEN)

    # This is the limit to which the account can go into debt.  The default is
    # zero which means the account cannot run a negative balance.  A 'source'
    # account will have no credit limit meaning it can transfer funds to other
    # accounts without limit.
    credit_limit = models.DecimalField(
        decimal_places=2,
        max_digits=12,
        default=Decimal("0.00"),
        null=True,
        blank=True,
    )
    balance = models.DecimalField(
        default=0,
        max_digits=15,
        decimal_places=2,
        editable=False,
        verbose_name=_("Balance"),
    )
    objects = PolymorphicManager()
    active = ActiveAccountManager()
    expired = ExpiredAccountManager()

    class Meta:
        db_table = "simpel_accounting_account"
        verbose_name = _("Account")
        verbose_name_plural = _("Accounts")

    @property
    def has_credit_limit(self):
        return self.credit_limit is not None

    @property
    def is_open(self):
        return self.status == Account.OPEN

    @property
    def is_closed(self):
        return self.status == Account.CLOSED

    @property
    def is_frozen(self):
        return self.status == Account.FROZEN

    def __str__(self):
        name = _("Anonymous")
        if self.name:
            name = self.name
        return "%s (%s)" % (name, self.account_type)

    def is_active(self):
        if self.start_date is None and self.end_date is None:
            return True
        now = timezone.now()
        if self.start_date and self.end_date is None:
            return now >= self.start_date
        if self.start_date is None and self.end_date:
            return now < self.end_date
        return self.start_date <= now < self.end_date

    def get_balance(self):
        # For performance, we keep a cached balance.  This can always be
        # recalculated from the account transactions.
        aggregates = self.entries.filter(status=Transaction.CONFIRMED).aggregate(sum=Sum("computed_amount"))
        sum = aggregates["sum"]
        balance = Decimal("0.00") if sum is None else sum
        return self.account_type.debit * balance

    def num_entries(self):
        return self.entries.filter(status=Transaction.CONFIRMED).count()

    def compute(self):
        self.balance = self.get_balance()
        self.last_modified = timezone.now()

    def clean(self):
        if self.linked_object_type is not None and self.linked_object is None:
            raise ValidationError({"linked_object_id": _("Reference not found.")})

    def save(self, *args, **kwargs):
        if self.code:
            self.code = self.code.upper()
        # Ensure the balance is always correct when saving
        self.compute()
        return super().save(*args, **kwargs)


class TransactionType(MPTTModel):
    parent = TreeForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="children",
    )
    code = models.CharField(
        unique=True,
        max_length=3,
        verbose_name=_("Code"),
        help_text=_("This unique code will used as Transaction Inner ID prefix eg: TRX.0202.001"),
    )
    name = models.CharField(
        max_length=255,
        verbose_name=_("Name"),
    )
    description = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        verbose_name=_("Description"),
    )

    class Meta:
        db_table = "simpel_accounting_transaction_type"
        verbose_name = _("Transaction Type")
        verbose_name_plural = _("Transaction Types")

    def __str__(self):
        return "%s: %s" % (self.code, self.name)

    def clean(self):
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError("Parent type cannot be self.")
            if parent.parent and parent.parent == self:
                raise ValidationError("Cannot have circular Parents.")


class Transaction(PolymorphicModel, NumeratorMixin):
    PENDING = "pending"
    CONFIRMED = "confirmed"
    CANCEL = "canceled"

    CHOICES = [
        (PENDING, _("Pending")),
        (CONFIRMED, _("Confirmed")),
        (CANCEL, _("Canceled")),
    ]
    type = TreeForeignKey(
        TransactionType,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="transactions",
        verbose_name=_("Type"),
    )
    status = models.CharField(
        max_length=15,
        choices=CHOICES,
        default=PENDING,
        verbose_name=_("Status"),
    )
    group = models.CharField(
        max_length=3,
        null=True,
        blank=True,
        verbose_name=_("Group"),
    )

    reference_type = models.ForeignKey(
        ContentType,
        null=True,
        blank=True,
        limit_choices_to={"model__in": ["salesorder", "invoice", "payment"]},
        on_delete=models.SET_NULL,
    )
    reference_id = models.CharField(
        max_length=255,
        verbose_name=_("Reference"),
    )
    reference = CustomGenericForeignKey(
        "reference_type",
        "reference_id",
        "inner_id",
    )

    note = models.CharField(
        max_length=LEN_LONG,
        null=True,
        blank=True,
        verbose_name=_("Memo"),
    )
    objects = TransactionManager()
    doc_prefix = "TRX"
    reset_mode = NumeratorReset.MONTHLY

    class Meta:
        db_table = "simpel_accounting_transaction"
        verbose_name = _("Transaction")
        verbose_name_plural = _("Transactions")
        index_together = ("created_at", "inner_id", "status")

    def __str__(self):
        return str(self.inner_id)

    def format_inner_id(self):
        """Inner ID final format"""
        form = [self.get_doc_prefix(), self.format_date(form="%m%y"), self.format_number()]
        inner_id = "{}.{}.{}".format(*form)
        return setattr(self, self.inner_id_field, inner_id)

    @transaction.atomic
    def confirm(self):
        if self.status != Transaction.PENDING:
            raise PermissionError(_("can't confirm transaction #%s") % self.inner_id)
        else:
            self.status = Transaction.CONFIRMED
            self.save()

    @transaction.atomic
    def cancel(self):
        self.status = Transaction.CANCEL
        self.save()

    def get_doc_prefix(self):
        if self.type is not None:
            return self.type.code
        return super().get_doc_prefix()

    def compute(self):
        for entry in self.entries.all():
            entry.save()
            entry.account.save()

    def save(self, *args, **kwargs):
        self.compute()
        return super().save(*args, **kwargs)


class EntryManager(models.Manager):
    def get_queryset(self):
        qs = super().get_queryset()
        return qs


class Entry(models.Model):
    DEBIT = 1
    CREDIT = -1
    FLOW = (
        (DEBIT, _("Debit")),
        (CREDIT, _("Credit")),
    )
    created_at = models.DateTimeField(
        default=timezone.now,
        editable=False,
        db_index=True,
    )
    trx = models.ForeignKey(
        Transaction,
        models.CASCADE,
        verbose_name=_("Transaction"),
        related_name="entries",
        db_index=True,
    )
    account = models.ForeignKey(
        Account,
        models.CASCADE,
        related_name="entries",
        db_index=True,
    )
    flow = models.IntegerField(choices=FLOW)
    amount = models.DecimalField(
        decimal_places=2,
        max_digits=12,
    )
    computed_amount = models.DecimalField(
        decimal_places=2,
        max_digits=12,
    )
    objects = EntryManager()
    status = models.CharField(
        max_length=15,
        choices=Transaction.CHOICES,
        default=Transaction.PENDING,
        verbose_name=_("Status"),
    )

    class Meta:
        db_table = "simpel_accounting_entry"
        index_together = ("id", "trx", "account")
        unique_together = ("trx", "account")
        verbose_name = _("Entry")
        verbose_name_plural = _("Entries")

    def __str__(self):
        return "Ref: %s, amount: %.2f" % (self.trx.reference, self.amount)

    def delete(self, *args, **kwargs):
        raise RuntimeError("Entries cannot be deleted")

    def reference(self):
        return self.trx.reference

    def note(self):
        return self.trx.note

    def debit(self):
        return Decimal("0.00") if self.flow == Entry.CREDIT else self.amount

    def credit(self):
        return Decimal("0.00") if self.flow == Entry.DEBIT else self.amount

    def get_computed_amount(self):
        # The sum of this field over the whole table should always be 0.
        # Debits should be positive while credits should be negative
        return self.amount * self.flow

    def compute(self):
        self.status = self.trx.status
        self.computed_amount = self.get_computed_amount()

    def save(self, *args, **kwargs):
        self.compute()
        return super().save(*args, **kwargs)



# Generated by Django 3.2.12 on 2022-03-23 17:47

from decimal import Decimal
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='TransactionType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(help_text='This unique code will used as Transaction Inner ID prefix eg: TRX.0202.001', max_length=3, unique=True, verbose_name='Code')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='Description')),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='simpel_journals.transactiontype')),
            ],
            options={
                'verbose_name': 'Transaction Type',
                'verbose_name_plural': 'Transaction Types',
                'db_table': 'simpel_accounting_transaction_type',
            },
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reg_number', models.PositiveIntegerField(blank=True, editable=False, null=True, verbose_name='Reg number')),
                ('inner_id', models.CharField(blank=True, editable=False, max_length=50, null=True, unique=True, verbose_name='Inner ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='created at')),
                ('status', models.CharField(choices=[('pending', 'Pending'), ('confirmed', 'Confirmed'), ('canceled', 'Canceled')], default='pending', max_length=15, verbose_name='Status')),
                ('group', models.CharField(blank=True, max_length=3, null=True, verbose_name='Group')),
                ('reference_id', models.CharField(max_length=255, verbose_name='Reference')),
                ('note', models.CharField(blank=True, max_length=255, null=True, verbose_name='Memo')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_simpel_journals.transaction_set+', to='contenttypes.contenttype')),
                ('reference_type', models.ForeignKey(blank=True, limit_choices_to={'model__in': ['salesorder', 'invoice', 'payment']}, null=True, on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.contenttype')),
                ('type', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='transactions', to='simpel_journals.transactiontype', verbose_name='Type')),
            ],
            options={
                'verbose_name': 'Transaction',
                'verbose_name_plural': 'Transactions',
                'db_table': 'simpel_accounting_transaction',
                'index_together': {('created_at', 'inner_id', 'status')},
            },
        ),
        migrations.CreateModel(
            name='AccountType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('debit', models.IntegerField(choices=[(1, 'Increase'), (-1, 'Decrease')], default=1)),
                ('code', models.CharField(blank=True, max_length=128, null=True, unique=True)),
                ('name', models.CharField(max_length=255)),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='simpel_journals.accounttype')),
            ],
            options={
                'verbose_name': 'Account Type',
                'verbose_name_plural': 'Account Types',
                'db_table': 'simpel_accounting_account_type',
            },
        ),
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Created at')),
                ('last_modified_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Created at')),
                ('start_date', models.DateTimeField(blank=True, null=True)),
                ('end_date', models.DateTimeField(blank=True, null=True)),
                ('code', models.CharField(blank=True, max_length=128, null=True, unique=True)),
                ('linked_object_id', models.IntegerField(blank=True, help_text='Linked instance primary key.', null=True)),
                ('name', models.CharField(blank=True, max_length=128, null=True)),
                ('description', models.TextField(blank=True, help_text='This text is shown to customers during checkout', null=True)),
                ('status', models.CharField(choices=[('Open', 'Open'), ('Frozen', 'Frozen'), ('Closed', 'Closed')], default='Open', max_length=32)),
                ('credit_limit', models.DecimalField(blank=True, decimal_places=2, default=Decimal('0.00'), max_digits=12, null=True)),
                ('balance', models.DecimalField(decimal_places=2, default=0, editable=False, max_digits=15, verbose_name='Balance')),
                ('account_type', mptt.fields.TreeForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts', to='simpel_journals.accounttype')),
                ('linked_object_type', models.ForeignKey(blank=True, help_text='Linked instance type', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='accounts', to='contenttypes.contenttype')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_simpel_journals.account_set+', to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Account',
                'verbose_name_plural': 'Accounts',
                'db_table': 'simpel_accounting_account',
            },
        ),
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False)),
                ('flow', models.IntegerField(choices=[(1, 'Debit'), (-1, 'Credit')])),
                ('amount', models.DecimalField(decimal_places=2, max_digits=12)),
                ('computed_amount', models.DecimalField(decimal_places=2, max_digits=12)),
                ('status', models.CharField(choices=[('pending', 'Pending'), ('confirmed', 'Confirmed'), ('canceled', 'Canceled')], default='pending', max_length=15, verbose_name='Status')),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='entries', to='simpel_journals.account')),
                ('trx', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='entries', to='simpel_journals.transaction', verbose_name='Transaction')),
            ],
            options={
                'verbose_name': 'Entry',
                'verbose_name_plural': 'Entries',
                'db_table': 'simpel_accounting_entry',
                'unique_together': {('trx', 'account')},
                'index_together': {('id', 'trx', 'account')},
            },
        ),
    ]



from django.contrib.contenttypes.models import ContentType

from .models import Account, AccountType
from .settings import journals_setting

names = journals_setting.NAMES


def get_or_create_account(
    name=None,
    type_name=None,
    code=None,
    account_model=None,
    linked_object=None,
):
    filters = dict()
    if account_model is None:
        account_model = Account
    if linked_object is not None:
        ctype = ContentType.objects.get_for_model(linked_object.__class__)
        filters["linked_object_type"] = ctype
        filters["linked_object_id"] = linked_object.id
    if type_name is not None:
        acc_type = AccountType.objects.get(name=type_name)
        filters["account_type"] = acc_type
    filters["code"] = code
    defaults = {"name": name}
    account, created = account_model.objects.get_or_create(**filters, defaults=defaults)
    return account, created


def get_partner_balance_account(partner):
    account, _ = get_or_create_account(
        name="%s (PDM)" % str(partner),
        code="%s.PDM" % partner.inner_id,
        type_name=names["PARTNER_BALANCE"],
        linked_object=partner,
    )
    return account


def get_partner_receivable_account(partner):
    account, _ = get_or_create_account(
        name="%s (PIU)" % str(partner),
        code="%s.PIU" % partner.inner_id,
        type_name=names["ACCOUNT_RECEIVABLE"],
        linked_object=partner,
    )
    return account


def get_cash_account(cash_gateway, type_name=None):
    code = "%s" % cash_gateway.id
    account, _ = get_or_create_account(
        name=str(cash_gateway),
        type_name=type_name or names["CASH"],
        code="CASH.%s" % (code.zfill(4)),
        linked_object=cash_gateway,
    )
    return account



from django.contrib import admin

# from django.utils.text import Truncator
from mptt.admin import DraggableMPTTAdmin
from polymorphic.admin import PolymorphicParentModelAdmin  # PolymorphicChildModelAdmin

from .models import Account, AccountType, Entry, Transaction, TransactionType


@admin.register(AccountType)
class AccountTypeAdmin(DraggableMPTTAdmin):
    list_display = ("tree_actions", "indented_title", "debit")  # Sane defaults.
    list_display_links = ("indented_title",)  # Sane defaults.


@admin.register(TransactionType)
class TransactionTypeAdmin(DraggableMPTTAdmin):
    list_display = ("tree_actions", "indented_title", "description")  # Sane defaults.
    list_display_links = ("indented_title",)  # Sane defaults.


@admin.register(Account)
class AccountAdmin(PolymorphicParentModelAdmin):
    child_models = [Account]
    list_filter = ["account_type"]
    search_fields = ["name"]
    list_display = [
        "code",
        "name",
        "balance",
        "status",
        "linked_object",
        "last_modified_at",
        "account_type",
    ]


@admin.register(Entry)
class EntryAdmin(admin.ModelAdmin):
    list_display = [
        "created_at",
        "trx",
        "account",
        "flow",
        "debit",
        "credit",
    ]
    autocomplete_fields = ["trx", "account"]


class EntryInline(admin.TabularInline):
    model = Entry
    extra = 1
    fields = ("account", "flow", "amount")
    autocomplete_fields = ["account"]


@admin.register(Transaction)
class TransactionAdmin(admin.ModelAdmin):
    inlines = [EntryInline]
    list_display = [
        "created_at",
        "reference_id",
        "reference",
        "note",
        "status",
    ]
    actions = ["confirm_transaction"]
    search_fields = ["note", "reference_id"]

    def confirm_transaction(self, request, queryset):
        for obj in queryset.all():
            obj.confirm()
